@@ -20,7 +20,7 @@
 
 from datetime import datetime, timedelta, date
 import os
-from pytz import timezone
+from pytz import timezone, UTC
 import sqlite3
 import sys
 from time import time
@@ -663,15 +663,27 @@ class TestSmsProvider(unittest.TestCase):
         self.assertNotIn(sms, list(self.provider.list_sms()))
 
     def test_sms_time_storage(self):
-        zones = ['Europe/London', # UTC+0(without DST), UTC+1(with DST)
-                 'Europe/Paris',  # UTC+1(without DST), UTC+2(with DST)
-                 'Asia/Jakarta']  # UTC+7(no DST adjustment in 2010)
+        tests = [
+            # UTC+0(without DST)
+            ('Europe/London', datetime(2015, 1, 17, 23, 35, 41)),
+            # UTC+1(with DST)
+            ('Europe/London', datetime(2015, 6, 17, 23, 35, 41)),
+            # UTC+1(without DST)
+            ('Europe/Paris', datetime(2015, 1, 17, 23, 35, 41)),
+            # UTC+2(with DST)
+            ('Europe/Paris', datetime(2015, 6, 17, 23, 35, 41)),
+            # UTC-5(without DST)
+            ('America/New_York', datetime(2015, 1, 17, 23, 35, 41)),
+            # UTC-4(with DST)
+            ('America/New_York', datetime(2015, 6, 17, 23, 35, 41)),
+            # UTC+7(no DST adjustment in 2010)
+            ('Asia/Jakarta', datetime(2015, 1, 17, 23, 35, 41))
+        ]
 
-        for tzstring in zones:
+        for tzstring, dt in tests:
             tz = timezone(tzstring)
 
-            # db resolution is secs
-            now = datetime.now(tz).replace(microsecond=0)
+            now = tz.localize(dt)
 
             sms = Message('+447917267410', tzstring, _datetime=now)
             sms = self.provider.add_sms(sms)
@@ -938,82 +950,198 @@ class TestUsageProvider(unittest.TestCase):
         self.provider.close()
 
     def test_add_usage_item(self):
-        now = get_tz_aware_now()
-        later = now + timedelta(minutes=30)
-        umts, bytes_recv, bytes_sent = True, 12345460, 12333211
-        item = self.provider.add_usage_item(now, later, bytes_recv,
-                                            bytes_sent, umts)
-        usage_items = self.provider.get_usage_for_day(date.today())
+        tz = timezone('Europe/London')
+        dt = tz.localize(datetime(2015, 1, 17, 15, 45))     # DST not applied
+
+        later = dt + timedelta(minutes=30)
+        item = self.provider.add_usage_item(dt, later, 12345460, 12333211, True)
+        usage_items = self.provider.get_usage_for_day(dt.date(), tz)
         self.assertIn(item, usage_items)
-        self.provider.delete_usage_item(item)
+
+    def test_add_usage_item_cet(self):
+        tz = timezone('Europe/Paris')
+        dt = tz.localize(datetime(2015, 1, 17, 15, 45))     # DST not applied
+        dt2 = datetime(2015, 1, 17, 14, 45, tzinfo=UTC)
+        dt3 = dt2 + timedelta(minutes=30)
+
+        later = dt + timedelta(minutes=30)
+
+        item = self.provider.add_usage_item(dt, later, 12345460, 12333211, True)
+        usage_items = self.provider.get_usage_for_day(dt.date(), tz)
+
+        self.assertEqual(dt2, dt.astimezone(UTC))
+
+        self.assertEqual(dt2, item.start_time.astimezone(UTC))
+        self.assertEqual(dt2, usage_items[0].start_time.astimezone(UTC))
+
+        self.assertEqual(dt3, item.end_time.astimezone(UTC))
+        self.assertEqual(dt3, usage_items[0].end_time.astimezone(UTC))
 
     def test_delete_usage_item(self):
-        now = get_tz_aware_now()
-        later = now + timedelta(minutes=60)
-        umts, bytes_recv, bytes_sent = True, 12345470, 12333212
-        item = self.provider.add_usage_item(now, later, bytes_recv,
-                                            bytes_sent, umts)
-        usage_items = self.provider.get_usage_for_day(date.today())
+        tz = timezone('Europe/London')
+        dt = tz.localize(datetime(2015, 1, 17, 15, 45))     # DST not applied
+
+        later = dt + timedelta(minutes=60)
+        item = self.provider.add_usage_item(dt, later, 12345470, 12333212, True)
+
+        # check it's there
+        usage_items = self.provider.get_usage_for_day(dt.date(), tz)
         self.assertIn(item, usage_items)
         self.provider.delete_usage_item(item)
+
         # now check that it is indeed gone
-        usage_items = self.provider.get_usage_for_day(date.today())
+        usage_items = self.provider.get_usage_for_day(dt.date(), tz)
         self.assertNotIn(item, usage_items)
 
-    def test_get_usage_for_day(self):
+    def _make_day_usage_items(self, dt):
         # add one usage item for today (45m)
-        now1 = get_tz_aware_now()
+        now1 = dt
         later1 = now1 + timedelta(minutes=45)
-        umts1, bytes_recv1, bytes_sent1 = True, 1200034, 124566
-        item1 = self.provider.add_usage_item(now1, later1, bytes_recv1,
-                                             bytes_sent1, umts1)
+        item1 = self.provider.add_usage_item(now1, later1, 100034, 14566, True)
+
         # add another usage item for today (17m)  55 minutes later
-        now2 = get_tz_aware_now() + timedelta(minutes=55)
-        later2 = now1 + timedelta(minutes=17)
-        umts2, bytes_recv2, bytes_sent2 = True, 12000, 1245
-        item2 = self.provider.add_usage_item(now2, later2, bytes_recv2,
-                                             bytes_sent2, umts2)
+        now2 = dt + timedelta(minutes=55)
+        later2 = now2 + timedelta(minutes=17)
+        item2 = self.provider.add_usage_item(now2, later2, 12000, 1245, True)
 
         # add another usage item for tomorrow (25m)
-        now3 = get_tz_aware_now() + timedelta(days=1)
+        now3 = dt + timedelta(days=1)
         later3 = now3 + timedelta(minutes=25)
-        umts3, bytes_recv3, bytes_sent3 = True, 14000, 1785
-        item3 = self.provider.add_usage_item(now3, later3, bytes_recv3,
-                                             bytes_sent3, umts3)
+        item3 = self.provider.add_usage_item(now3, later3, 14000, 1785, True)
+
+        return (item1, item2, item3)
+
+    def test_get_usage_for_day_bst_within(self):
+        tz = timezone('Europe/London')
+        dt = tz.localize(datetime(2015, 6, 17, 15, 30))     # DST applied
+
+        item1, item2, item3 = self._make_day_usage_items(dt)
 
         # now get the usage for today
-        today_items = self.provider.get_usage_for_day(date.today())
+        today_items = self.provider.get_usage_for_day(dt.date(), tz)
         self.assertIn(item1, today_items)
         self.assertIn(item2, today_items)
         self.assertNotIn(item3, today_items)
+
         # get the usage for tomorrow
-        tomorrow = date.today() + timedelta(days=1)
-        tomorrow_items = self.provider.get_usage_for_day(tomorrow)
+        tomorrow = dt.date() + timedelta(days=1)
+        tomorrow_items = self.provider.get_usage_for_day(tomorrow, tz)
         self.assertNotIn(item1, tomorrow_items)
         self.assertNotIn(item2, tomorrow_items)
         self.assertIn(item3, tomorrow_items)
 
-        # leave it as we found it
-        for i in [item1, item2, item3]:
-            self.provider.delete_usage_item(i)
+    def test_get_usage_for_day_bst_overlap(self):
+        tz = timezone('Europe/London')
+        dt = tz.localize(datetime(2015, 6, 17, 23, 30))     # DST applied
+
+        item1, item2, item3 = self._make_day_usage_items(dt)
+
+        # now get the usage for today, but all items have end time tomorrow
+        today_items = self.provider.get_usage_for_day(dt.date(), tz)
+        self.assertNotIn(item1, today_items)
+        self.assertNotIn(item2, today_items)
+        self.assertNotIn(item3, today_items)
+
+        # get the usage for tomorrow
+        tomorrow = dt.date() + timedelta(days=1)
+        tomorrow_items = self.provider.get_usage_for_day(tomorrow, tz)
+        self.assertIn(item1, tomorrow_items)
+        self.assertIn(item2, tomorrow_items)
+        self.assertIn(item3, tomorrow_items)
+
+    def test_get_usage_for_day_est_within(self):
+        tz = timezone('America/New_York')
+        dt = tz.localize(datetime(2015, 1, 17, 15, 30))     # DST not applied
+
+        item1, item2, item3 = self._make_day_usage_items(dt)
+
+        # now get the usage for today
+        today_items = self.provider.get_usage_for_day(dt.date(), tz)
+        self.assertIn(item1, today_items)
+        self.assertIn(item2, today_items)
+        self.assertNotIn(item3, today_items)
+
+        # get the usage for tomorrow
+        tomorrow = dt.date() + timedelta(days=1)
+        tomorrow_items = self.provider.get_usage_for_day(tomorrow, tz)
+        self.assertNotIn(item1, tomorrow_items)
+        self.assertNotIn(item2, tomorrow_items)
+        self.assertIn(item3, tomorrow_items)
+
+    def test_get_usage_for_day_est_overlap(self):
+        tz = timezone('America/New_York')
+        dt = tz.localize(datetime(2015, 1, 17, 23, 30))     # DST not applied
+
+        item1, item2, item3 = self._make_day_usage_items(dt)
+
+        # now get the usage for today, but all items have end time tomorrow
+        today_items = self.provider.get_usage_for_day(dt.date(), tz)
+        self.assertNotIn(item1, today_items)
+        self.assertNotIn(item2, today_items)
+        self.assertNotIn(item3, today_items)
+
+        # get the usage for tomorrow
+        tomorrow = dt.date() + timedelta(days=1)
+        tomorrow_items = self.provider.get_usage_for_day(tomorrow, tz)
+        self.assertIn(item1, tomorrow_items)
+        self.assertIn(item2, tomorrow_items)
+        self.assertIn(item3, tomorrow_items)
+
+    def test_get_usage_for_day_gmt_within(self):
+        tz = timezone('Europe/London')
+        dt = tz.localize(datetime(2015, 1, 17, 15, 30))     # DST not applied
+
+        item1, item2, item3 = self._make_day_usage_items(dt)
+
+        # now get the usage for today
+        today_items = self.provider.get_usage_for_day(dt.date(), tz)
+        self.assertIn(item1, today_items)
+        self.assertIn(item2, today_items)
+        self.assertNotIn(item3, today_items)
+
+        # get the usage for tomorrow
+        tomorrow = dt.date() + timedelta(days=1)
+        tomorrow_items = self.provider.get_usage_for_day(tomorrow, tz)
+        self.assertNotIn(item1, tomorrow_items)
+        self.assertNotIn(item2, tomorrow_items)
+        self.assertIn(item3, tomorrow_items)
+
+    def test_get_usage_for_day_gmt_overlap(self):
+        tz = timezone('Europe/London')
+        dt = tz.localize(datetime(2015, 1, 17, 23, 30))     # DST not applied
+
+        item1, item2, item3 = self._make_day_usage_items(dt)
+
+        # now get the usage for today, but all items have end time tomorrow
+        today_items = self.provider.get_usage_for_day(dt.date(), tz)
+        self.assertNotIn(item1, today_items)
+        self.assertNotIn(item2, today_items)
+        self.assertNotIn(item3, today_items)
+
+        # get the usage for tomorrow
+        tomorrow = dt.date() + timedelta(days=1)
+        tomorrow_items = self.provider.get_usage_for_day(tomorrow, tz)
+        self.assertIn(item1, tomorrow_items)
+        self.assertIn(item2, tomorrow_items)
+        self.assertIn(item3, tomorrow_items)
 
     def test_get_usage_for_month(self):
-        current_month = date.today().month
-        current_year = date.today().year
+        tz = timezone('Europe/London')
+        dt = tz.localize(datetime(2015, 1, 17, 15, 30))     # DST not applied
+
+        current_month = dt.month
+        current_year = dt.year
+
         # add one usage item for day 12 of this month (45m)
-        now1 = datetime(date.today().year, current_month, 12, 13, 10,
-                        tzinfo=timezone('UTC'))
+        now1 = tz.localize(datetime(current_year, current_month, 12, 13, 10))
         later1 = now1 + timedelta(minutes=45)
-        umts1, bytes_recv1, bytes_sent1 = True, 1200034, 124566
-        item1 = self.provider.add_usage_item(now1, later1, bytes_recv1,
-                                             bytes_sent1, umts1)
+        item1 = self.provider.add_usage_item(now1, later1, 100034, 12566, True)
+
         # add another usage item for day 13 of this month (17m)
-        now2 = datetime(date.today().year, current_month, 13, 15, 10,
-                        tzinfo=timezone('UTC'))
-        later2 = now1 + timedelta(minutes=17)
-        umts2, bytes_recv2, bytes_sent2 = True, 12000, 1245
-        item2 = self.provider.add_usage_item(now2, later2, bytes_recv2,
-                                             bytes_sent2, umts2)
+        now2 = tz.localize(datetime(current_year, current_month, 13, 15, 10))
+        later2 = now2 + timedelta(minutes=17)
+        item2 = self.provider.add_usage_item(now2, later2, 12000, 1245, True)
+
         # add another usage item for next month
         if current_month < 12:
             month = current_month + 1
@@ -1023,71 +1151,62 @@ class TestUsageProvider(unittest.TestCase):
             year = current_year + 1
 
         # next month at 6.50am (25m)
-        now3 = datetime(year, month, 2, 6, 50, tzinfo=timezone('UTC'))
+        now3 = tz.localize(datetime(year, month, 2, 6, 50))
         later3 = now3 + timedelta(minutes=25)
-        umts3, bytes_recv3, bytes_sent3 = True, 14000, 1785
-        item3 = self.provider.add_usage_item(now3, later3, bytes_recv3,
-                                             bytes_sent3, umts3)
+        item3 = self.provider.add_usage_item(now3, later3, 14000, 1785, True)
+
         # now get the usage for this month
-        this_month_items = self.provider.get_usage_for_month(now1.date())
+        this_month_items = self.provider.get_usage_for_month(now1.date(), tz)
         self.assertIn(item1, this_month_items)
         self.assertIn(item2, this_month_items)
         self.assertNotIn(item3, this_month_items)
+
         # now get the usage for next month
-        next_month_items = self.provider.get_usage_for_month(now3)
+        next_month_items = self.provider.get_usage_for_month(now3.date(), tz)
         self.assertNotIn(item1, next_month_items)
         self.assertNotIn(item2, next_month_items)
         self.assertIn(item3, next_month_items)
-        # leave it as we found it
-        for i in [item1, item2, item3]:
-            self.provider.delete_usage_item(i)
 
     def test_get_total_usage(self):
-        current_month = date.today().month
-        current_year = date.today().year
-        # add one usage item for day 12 of this month (45m)
-        now1 = datetime(current_year, current_month, 12, 13, 10,
-                        tzinfo=timezone('UTC'))
-        later1 = now1 + timedelta(minutes=45)
-        umts1, bytes_recv1, bytes_sent1 = True, 1200034, 124566
-        item1 = self.provider.add_usage_item(now1, later1, bytes_recv1,
-                                             bytes_sent1, umts1)
-        # add another usage item for day 13 of this month (17m), one year ago
-        now2 = datetime(current_year - 1, current_month, 13, 15, 10,
-                        tzinfo=timezone('UTC'))
-        later2 = now1 + timedelta(minutes=17)
-        umts2, bytes_recv2, bytes_sent2 = True, 12000, 1245
-        item2 = self.provider.add_usage_item(now2, later2, bytes_recv2,
-                                             bytes_sent2, umts2)
+        tz = timezone('Europe/London')
+        dt = tz.localize(datetime(2015, 1, 17, 15, 30))     # DST not applied
 
-        items = self.provider.get_total_usage()
+        current_month = dt.month
+        current_year = dt.year
+
+        # add one usage item for day 12 of this month (45m)
+        now1 = tz.localize(datetime(current_year, current_month, 12, 13, 10))
+        later1 = now1 + timedelta(minutes=45)
+        item1 = self.provider.add_usage_item(now1, later1, 100034, 12566, True)
+
+        # add another usage item for day 13 of this month (17m), one year ago
+        now2 = tz.localize(datetime(current_year - 1, current_month,
+                                    13, 15, 10))
+        later2 = now2 + timedelta(minutes=17)
+        item2 = self.provider.add_usage_item(now2, later2, 12000, 1245, True)
+
+        items = self.provider.get_total_usage(tz=tz)
         self.assertIn(item1, items)
         self.assertIn(item2, items)
-        # leave it as we found it
-        for i in [item1, item2]:
-            self.provider.delete_usage_item(i)
 
     def test_get_total_usage_passing_a_date(self):
-        current_month = date.today().month
-        current_year = date.today().year
-        # add one usage item for day 12 of this month (45m)
-        now1 = datetime(current_year, current_month, 12, 13, 10,
-                        tzinfo=timezone('UTC'))
-        later1 = now1 + timedelta(minutes=45)
-        umts1, bytes_recv1, bytes_sent1 = True, 1200034, 124566
-        item1 = self.provider.add_usage_item(now1, later1, bytes_recv1,
-                                             bytes_sent1, umts1)
-        # add another usage item for day 13 of this month (17m), one year ago
-        now2 = datetime(current_year - 1, current_month, 13, 15, 10,
-                        tzinfo=timezone('UTC'))
-        later2 = now1 + timedelta(minutes=17)
-        umts2, bytes_recv2, bytes_sent2 = True, 12000, 1245
-        item2 = self.provider.add_usage_item(now2, later2, bytes_recv2,
-                                             bytes_sent2, umts2)
+        tz = timezone('Europe/London')
+        dt = tz.localize(datetime(2015, 1, 17, 15, 30))     # DST not applied
 
-        items = self.provider.get_total_usage(now1.date())
+        current_month = dt.month
+        current_year = dt.year
+
+        # add one usage item for day 12 of this month (45m)
+        now1 = tz.localize(datetime(current_year, current_month, 12, 13, 10))
+        later1 = now1 + timedelta(minutes=45)
+        item1 = self.provider.add_usage_item(now1, later1, 100034, 12566, True)
+
+        # add another usage item for day 13 of this month (17m), one year ago
+        now2 = tz.localize(datetime(current_year - 1, current_month,
+                                    13, 15, 10))
+        later2 = now2 + timedelta(minutes=17)
+        item2 = self.provider.add_usage_item(now2, later2, 12000, 1245, True)
+
+        items = self.provider.get_total_usage(now1.date(), tz=tz)
         self.assertIn(item1, items)
         self.assertNotIn(item2, items)
-        # leave it as we found it
-        for i in [item1, item2]:
-            self.provider.delete_usage_item(i)
