@@ -166,7 +166,7 @@ end;
 
 NETWORKS_SCHEMA = """
 create table network_info(
-    id text primary key,
+    id integer primary key,
     name text,
     country text);
 
@@ -286,10 +286,15 @@ sqlite3.register_adapter(datetime.datetime, adapt_datetime)
 sqlite3.register_converter("datetime", convert_datetime)
 
 
-def date_to_datetime(date):
-    # takes a date which is in local time and returns a timezone aware datetime
-    timestamp = mktime(date.timetuple())
-    return datetime.datetime.fromtimestamp(timestamp, timezone('UTC'))
+def date_to_datetime(date, tz=None):
+    # takes a date which is in local time (or optionally has timezone) and
+    # returns a timezone aware datetime
+
+    if tz is None:
+        timestamp = mktime(date.timetuple())
+        return datetime.datetime.fromtimestamp(timestamp, timezone('UTC'))
+
+    return tz.localize(datetime.datetime(date.year, date.month, date.day, 0, 0))
 
 
 # common classes
@@ -385,11 +390,12 @@ class UsageProvider(DBProvider):
         c = self.conn.cursor()
         c.execute("delete from usage where id=?", (item.index,))
 
-    def get_usage_for_day(self, day):
+    def get_usage_for_day(self, day, tz=None):
         """
         Returns all `UsageItem` for ``day``
 
         :type day: ``datetime.date``
+        :type tz: ``pytz.tzfile``
         """
         c = self.conn.cursor()
 
@@ -397,13 +403,13 @@ class UsageProvider(DBProvider):
             raise ValueError("Don't know what to do with %s" % day)
 
         tomorrow = day + datetime.timedelta(days=1)
-        args = (date_to_datetime(day), date_to_datetime(tomorrow))
-        c.execute("select * from usage where start_time >= ? and end_time < ?",
+        args = (date_to_datetime(day, tz), date_to_datetime(tomorrow, tz))
+        c.execute("select * from usage where end_time >= ? and end_time < ?",
                   args)
 
         return [UsageItem.from_row(row) for row in c.fetchall()]
 
-    def get_usage_for_month(self, month):
+    def get_usage_for_month(self, month, tz=None):
         """
         Returns all ``UsageItem`` for ``month``
 
@@ -424,13 +430,13 @@ class UsageProvider(DBProvider):
 
         first_next_month_day = month.replace(day=1, month=_month, year=_year)
 
-        args = (date_to_datetime(first_current_month_day),
-                date_to_datetime(first_next_month_day))
-        c.execute("select * from usage where start_time >= ? and start_time < ?",
+        args = (date_to_datetime(first_current_month_day, tz),
+                date_to_datetime(first_next_month_day, tz))
+        c.execute("select * from usage where end_time >= ? and end_time < ?",
                   args)
         return [UsageItem.from_row(row) for row in c.fetchall()]
 
-    def get_total_usage(self, month=None):
+    def get_total_usage(self, month=None, tz=None):
         c = self.conn.cursor()
 
         if month is None:
@@ -440,8 +446,8 @@ class UsageProvider(DBProvider):
             if not isinstance(month, datetime.date):
                 raise ValueError("Don't know what to do with %s" % month)
 
-            sql = "select * from usage where start_time >= ?"
-            c.execute(sql, (date_to_datetime(month),))
+            sql = "select * from usage where end_time >= ?"
+            c.execute(sql, (date_to_datetime(month, tz),))
 
         return [UsageItem.from_row(row) for row in c.fetchall()]
 
@@ -614,7 +620,14 @@ class NetworkProvider(DBProvider):
         c = self.conn.cursor()
 
         for network in networks:
-            for netid in network.netid:
+            for snetid in network.netid:
+
+                # DB field is int not string
+                try:
+                    netid = int(snetid)
+                except (ValueError, TypeError):
+                    continue
+
                 # check if this network object exists in the database
                 c.execute("select 1 from network_info where id=?", (netid,))
                 try:
@@ -706,7 +719,12 @@ class NetworkProvider(DBProvider):
             for networkid in getElementsByTagNameNoDescend(gsm, "network-id"):
                 mcc = getAttributeValue(networkid, "mcc")
                 mnc = getAttributeValue(networkid, "mnc")
-                netid = mcc + mnc
+
+                # DB field is int not string
+                try:
+                    netid = int(mcc + mnc)
+                except (ValueError, TypeError):
+                    continue
 
                 # check if this network object exists in the database
                 c.execute("select 1 from network_info where id=?", (netid,))
